<?php
/**
 * Class BaixaPagar
 * @category PHP
 * @package Financeiro\Logic\BaixaPagar
 * @author   leonardooliveira <leonardo.oliveira@teknisa.com>
 * @license  http://erp.pebbian.com Teknisa
 * @link     http://erp.pebbian.com
 */
namespace Financeiro\Logic\BaixaPagar;

use Financeiro\Entities\Ajctpagar;
use \Financeiro\Entities\Baixapaga;
use Financeiro\Entities\Ajbolpag;
use Financeiro\Entities\Movfluxo;
use Fiscal\Entities\Contapagar;
use \Fiscal\Entities\Boletpagar;
use \Geral\Util\DateUtil;
use \Geral\Util\Ambiente;
use \Setforms2\Shared\Registry;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use \Geral\Util\NovoCodigo;
use Financeiro\Entities\Movbancario;
use Financeiro\Entities\Adiantapagar;
use Financeiro\Entities\Cnabpag;
use \Setforms2\Shared\ORM\DateTime;
use Indeterminado\Entities\Devolucaopag;
use Fiscal\Entities\Notafiscal;
use \Financeiro\Logic\AtualizaFluxodeCaixa\FluxodeCaixa;
use Financeiro\Entities\Cheque;
/**
 * Class BaixaPagar
 * @category PHP
 * @package Financeiro\Logic\BaixaPagar
 * @author   leonardooliveira <leonardo.oliveira@teknisa.com>
 * @license  http://erp.pebbian.com Teknisa
 * @link     http://erp.pebbian.com
 */
class BaixaPagar
{
    const PREFIXO_CONTADOR_CONTABILIZACAO = 'LANCCONTABIL';

    const ID_CNAB_N_ENVIADO = '00';

    public static function insereBaixa(
        $nrorg, $cdempresa, $cdreducont, $dtbaixa, $dtlanctbaixa, $cdoperador, $idstatusbaixa, $vrbaixa,
        $cdtpbaixapag, $dsbaixapag, $nrcheque, $cdfilial, $nrmovbanc
    ){
        try {
            $baixa = new Baixapaga();
            $baixa->setNrbaixapag(NovoCodigo::geraCodigo($nrorg, 'BAIXAPAGA' . $cdfilial));
            $baixa->setCdempresa($cdempresa);
            $baixa->setCdreducont($cdreducont);
            $baixa->setDtlancbaixap($dtlanctbaixa);
            $baixa->setCdoperador($cdoperador);
            $baixa->setDtbaixapag(self::formataData($dtbaixa));
            $baixa->setIdstatusbpag($idstatusbaixa);
            $baixa->setVrbaixapag($vrbaixa);
            $baixa->setIdstatusctbpag('P');
            $baixa->setCdtpbaixapag($cdtpbaixapag);
            $baixa->setDsbaixapag($dsbaixapag);
            $baixa->setIdstemicheq('N');
            $baixa->setNrorg($nrorg);
            $baixa->setDtinclusao(DateUtil::getDataAtual());
            $baixa->setNrcheque($nrcheque);
            $baixa->setIdativo('S');
            $baixa->setCdfilial($cdfilial);
            $baixa->setNrmovbanc($nrmovbanc);
            $baixa->persist();

            Registry::getEntityManager()->flush();
            return $baixa->getNrbaixapag();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Método utilizado para atualizar a Entidade de Boleto a Pagar
     * @param Int $boletos    Boletos
     * @param Int $nrbaixapag Baixa
     * @param Int $status     Status
     * @throws \Exception
     * @return int
     */
    public static function updateBoletoPag($boletos, $nrbaixapag, $cdfilial, $status)
    {
        try {
            $cont = count($boletos['NRLANBOLPAG']);

            $valor = 0;

            for ($i = 0; $i < $cont; $i++) {
                /**
                 * Método
                 * @var $object Boletpagar
                 */
                $object = Boletpagar::getObjectByWhere('NRORG = ? AND CDFILIAL = ? AND NRLANBOLPAG = ?', array($boletos['NRORG'][$i], $boletos['CDFILIAL'][$i], $boletos['NRLANBOLPAG'][$i]));
                $object->setNrbaixapag($nrbaixapag);
                $object->setIdstatbpag($status);
                $object->setCdfilibaipag($cdfilial);
                $valor += $object->getVrfinalbpag();
                $object->persist();
            }
            return ($valor);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Método utilizado para atualizar a Entidade de Baixa a pagar
     * @param Int    $nrbaixapag Baixa
     * @param Int    $cdfilial   Filial
     * @param String $valor      Valor
     * @param Int    $nrmovbanc  Movimentação Bancária
     * @param Int    $nrcheque   Número do cheque
     * @throws \Exception
     * @return null
     */
    public static function updateBaixapaga($cdfilial, $nrbaixapag, $nrmovbanc, $valor, $nrcheque = NULL)
    {
        try {
            /**
             * Método utilizado para buscar o boleto
             * @var $object Baixapaga
             */
            $object = Baixapaga::getObjectByWhere('NRORG = ? AND CDFILIAL = ? AND NRBAIXAPAG = ? ', array(Ambiente::getNrorg(), $cdfilial, $nrbaixapag));
            $object->setVrbaixapag($valor);
            if($nrmovbanc)
                $object->setNrmovbanc($nrmovbanc);
            $object->setNrcheque($nrcheque);
            $object->persist();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $idstatpag
     * @param $dtmovfluxo
     * @param $idmovfluxo
     * @throws \Exception
     */
    public static function updateContasPagar($cdfilial, $nrlanbolpag, $idstatpag, $dtmovfluxo = null, $idmovfluxo){
        try {

            /** @var $conta Contapagar */
            $contas = Contapagar::findByNrorgAndCdfilibpagAndNrlanbolpag(
                Ambiente::getNrorg(), $cdfilial, $nrlanbolpag
            );

            if($dtmovfluxo)
                $dtmovfluxo = self::formataData($dtmovfluxo);

            foreach($contas as $conta){

                if($idmovfluxo == 'E')
                    $dtmovfluxo = $conta->getDtatuavenpag();
                $conta->setIdstatpag($idstatpag);
                $conta->setIdmovfluxo($idmovfluxo);
                $conta->setDtmovfluxo($dtmovfluxo);
                $conta->persist();
            }

            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Método utilizado para contabilizar a Baixa a Pagar
     * @param Int $cdempresa Empresa
     * @param Int $cdfilial  Empresa
     * @param Int $nrbaixa   Baixa
     * @return mixed
     * @throws \Exception
     */
    public static function contabilizaBaixaPaga($cdempresa, $cdfilial, $nrbaixa)
    {
        try {
            $nrlancctb = '';
            $procedure = new Procedure('CTB_BAIXA_PAG');
            $procedure->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $cdempresa, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRBAIXA', Param::PARAM_INPUT, $nrbaixa, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, $nrlancctb, Param::PARAM_TYPE_STR, 12));
            $nrlancctb = $procedure->call();
            return $nrlancctb['P_NRLANCCTB'];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Método utilizado para gerar a Movimentação Bancária
     * @param Int $cdreducont Conta
     * @param DateTime $dtmovbanc  Data
     * @param Int $vrmovbanc  Valor
     * @param null $nrdocmbanc
     * @throws \Exception
     * @return int|string
     */
    public static function geraMovimentacao($cdreducont, $dtmovbanc, $vrmovbanc, $nrdocmbanc = null)
    {
        try {
            $nrmovbanc = NovoCodigo::geraCodigo(Ambiente::getNrorg(), 'MOVBANCARIO' . $cdreducont);

            $movbancario = new Movbancario;
            $movbancario->setCdreducont($cdreducont);
            $movbancario->setNrmovbanc($nrmovbanc);
            $dtmovbanc = self::formataData($dtmovbanc);
            $movbancario->setDtmovbanc($dtmovbanc);
            $movbancario->setIdmovbanc('D');
            $movbancario->setDshismbanc('BaixaPag');
            $movbancario->setVrmovbanc($vrmovbanc);
            $movbancario->setIdorigmbanc('P');
            $movbancario->setIdstatusctbbanc('P');
            $movbancario->setNrorg(Ambiente::getNrorg());
            $movbancario->setDtinclusao(DateUtil::getDataAtual());
            $movbancario->setNrdocmbanc($nrdocmbanc);
            // Status de conciliação com o Extrato Bancário - 0 Não Conciliado
            $movbancario->setIdstconext('0');
            $movbancario->persist();

            Registry::getEntityManager()->flush();

            return $nrmovbanc;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Método utilizado para formatar a data
     * @param  DateTime $data Data
     * @return \Setforms2\Shared\ORM\DateTime
     */
    public static function formataData($data)
    {
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

    /**
     * Método utilizado para verifica se existe o Adiantamento
     * @param Int $cdempresa  Empresa
     * @param Int $cdforneced Fornecedor
     * @return array
     */
    public static function verificaAdiantamento($cdempresa, $cdforneced)
    {
        /**
         * Método
         * @var $adiantapagarRepository \Financeiro\Repositories\Adiantapagar
         */
        $adiantapagarRepository = Adiantapagar::getRepository();
        $adiantamentos = $adiantapagarRepository->verificaAdiantamento($cdempresa, $cdforneced);
        return $adiantamentos;

    }


    /**
     * Método utilizado para verificar se existe Devolucao
     * @param Int $cdempresa  Empresa
     * @param Int $cdforneced Fornecedor
     * @return array
     */
    public static function verificaDevolucao($cdempresa, $cdforneced)
    {
        /**
         * Método
         * @var $devoluPagarRepository \Financeiro\Repositories\Adiantapagar
         */
        $devoluPagarRepository = Adiantapagar::getRepository();
        $devolucoes = $devoluPagarRepository->verificaDevolucao($cdempresa, $cdforneced);
        return $devolucoes;

    }

    /**
     * Método utilizado para Adiantar o boleto
     * @param Boletpagar $boleto            Boletos
     * @param Boletpagar $valoradiantamento Valor
     * @param Boletpagar $codigoajuste      Codigo Ajuste
     * @param Boletpagar $adiantamentos     Adiantamentos Marcados
     * @throws \Exception
     * @return null
     */
    public static function confirmaAdiantamento(Boletpagar $boleto, $valoradiantamento, $codigoajuste, $adiantamentos)
    {
        try {
            $connection = Registry::getEntityManager();
            $connection->beginTransaction();
            /**
             * Método
             * @@var $boleto Boletpagar
             */
            // Se possuir número de remessa cancela a mesma
            if ((bool)$boleto->getNrcnabpagrem() === true)
                self::cancelaRemessa($boleto);

            //Insere Ajuste
            $ajuste = new Ajbolpag();
            $ajuste->setCdajuctpag($codigoajuste);
            $ajuste->setVrajbolpag($valoradiantamento);
            $ajuste->setCdfilial($boleto->getCdfilial());
            $ajuste->setNrlanbolpag($boleto->getNrlanbolpag());
            $ajuste->setNrorg($boleto->getNrorg());
            $ajuste->setDtinclusao(DateUtil::getDataAtual());
            $ajuste->persist();
            Registry::getEntityManager()->flush();

            // Insere Adiantamento
            foreach ($adiantamentos as $item) {
                $adiantapagar = new Adiantapagar();
                $novocodigo = NovoCodigo::geraCodigo(Ambiente::getNrorg(), 'ADIANTAPAGAR');
                $adiantapagar->setNrorg(Ambiente::getNrorg());
                $adiantapagar->setNradiantapagar($novocodigo);
                $adiantapagar->setCdfiliajbolp($boleto->getCdfilial());
                $adiantapagar->setNrlanbolpag($boleto->getNrlanbolpag());
                $adiantapagar->setCdajuctpag($codigoajuste);
                $adiantapagar->setCdfilial($item['CDFILIAL']);
                $adiantapagar->setNrlanctonf($item['NRLANCTONF']);
                $adiantapagar->setNrseqvenpag($item['NRSEQVENPAG']);
                $adiantapagar->setNrparcelapag($item['NRPARCELAPAG']);
                $adiantapagar->setVradiantapag($item['VRDESCONTO']);
                $adiantapagar->setDtinclusao(DateUtil::getDataAtual());
                $adiantapagar->persist();

            }

            $boleto->setVrfinalbpag($boleto->getVrfinalbpag() - $valoradiantamento);
            $boleto->persist();
            Registry::getEntityManager()->flush();

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;

        }

    }

    /**
     * Método utilizado para Devolver o boleto
     * @param Boletpagar $boleto         Boleto
     * @param Int        $valordevolucao Valor
     * @param Int        $codigoajuste   Ajuste
     * @param Array      $devolucoes     Devolucoes
     * @throws \Exception
     * @return null
     */
    public static function confirmaDevolucao(Boletpagar $boleto, $valordevolucao, $codigoajuste, $devolucoes)
    {
        try {
            $connection = Registry::getEntityManager();
            $connection->beginTransaction();
            /**
             * Método
             * @@var $boleto Boletpagar
             */
            // Se possuir número de remessa cancela a mesma
            if ((bool)$boleto->getNrcnabpagrem() === true)
                self::cancelaRemessa($boleto);

            //Insere Ajuste
            $ajuste = new Ajbolpag();
            $ajuste->setCdajuctpag($codigoajuste);
            $ajuste->setVrajbolpag($valordevolucao);
            $ajuste->setCdfilial($boleto->getCdfilial());
            $ajuste->setNrlanbolpag($boleto->getNrlanbolpag());
            $ajuste->setNrorg($boleto->getNrorg());
            $ajuste->setDtinclusao(DateUtil::getDataAtual());
            $ajuste->persist();
            Registry::getEntityManager()->flush();

            // Insere Adiantamento
            foreach ($devolucoes as $item) {
                $devolucaorec = new Devolucaopag();
                $devolucaorec->setCdfilibolpag($boleto->getCdfilial());
                $devolucaorec->setNrlanbolpag($boleto->getNrlanbolpag());
                $devolucaorec->setCdajuctpag($codigoajuste);
                $devolucaorec->setCdfilial($item['FILIDEVO']);
                $devolucaorec->setNrlanctonf($item['NRLANCTONF']);
                $devolucaorec->setVrdevolucaopag($item['VRDESCONTO']);
                $devolucaorec->setNrorg(Ambiente::getNrorg());
                $devolucaorec->setDtinclusao(DateUtil::getDataAtual());
                $devolucaorec->persist();
                $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($item['FILIDEVO'], $item['NRLANCTONF'], $item['NRORG']);
                $notafiscal->setVrdevolnfutl($notafiscal->getVrdevolnfutl() + $item['VRDESCONTO']);
                $notafiscal->setNrnotadevol($item['VRDESCONTO']);
                $notafiscal->persist();
            }

            $boleto->setVrfinalbpag($boleto->getVrfinalbpag() - $valordevolucao);
            $boleto->persist();
            Registry::getEntityManager()->flush();

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;

        }

    }

    /**
     * Metodo utilizado para cancelar a Remessa
     * @param Boletpagar $boleto Boletos
     * @throws \Exception
     * @return null
     */
    public static function cancelaRemessa(Boletpagar $boleto)
    {
        try{
            //Se o boleto estiver ligado a uma remessa
            if ((bool)$boleto->getNrcnabpagrem() === true) {
                //Apaga o vínculo do boleto com a remessa e coloca o Status de "Não enviado"
                $boleto->setIdsitucnabp(self::ID_CNAB_N_ENVIADO);
                $boleto->persist();
                /**
                 * Método
                 * @var $cnab Cnabpag
                 */
                //Atualiza o valor e a quantidade de boletos da remessa ao qual o boleto pertencia
                $cnab = Cnabpag::findOneByCdempresaAndNrcnabpagAndIdtipcnabpagAndNrorg(
                    $boleto->getCdempresa(),
                    $boleto->getNrcnabpagrem(),
                    '0',
                    $boleto->getNrorg()
                );
                $cnab->setVrtotalcnabpag(round($cnab->getVrtotalcnabpag() - $boleto->getVrbolpag()), 2);
                $cnab->setQtbolcnabpag($cnab->getQtbolcnabpag() -1);
                $cnab->persist();
            }

        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param $cdfilial
     * @param $nrbaixapag
     * @throws \Exception
     */
    public static function baixaReal($cdfilial, $nrbaixapag){

        try{

            /** @var $baixapaga Baixapaga */
            $baixapaga = Baixapaga::findOneByNrbaixapagAndCdfilialAndNrorg($nrbaixapag, $cdfilial, \Ambiente::getNrorg());

            /** @var $boletos Boletpagar  */
            $boletos = Boletpagar::findByNrbaixapagAndCdfilibaipagAndNrorg($nrbaixapag, $cdfilial, \Ambiente::getNrorg());

                       $nrmovbanc = null;
            //Insere Movimento Bancário
            if($baixapaga->getVrbaixapag() != 0){

                $nrmovbanc = self::geraMovimentacao($baixapaga->getCdreducont(),
                                       $baixapaga->getDtbaixapag(),
                                       $baixapaga->getVrbaixapag(),$baixapaga->getNrcheque()
                                       );
            }

            // Atualiza o saldo da conta corrente
            FluxodeCaixa::atualizaSaldoContCorr($baixapaga->getCdreducont(),$baixapaga->getDtbaixapag(),
                                                $baixapaga->getVrbaixapag(),'D');

            /** @var $boleto Boletpagar */
            foreach($boletos as $boleto){
                self::atualizaContas($boleto->getCdfilial(),$boleto->getNrlanbolpag(),$baixapaga->getDtbaixapag(),
                                    'E','I');

                $boleto->setIdstatbpag(Boletpagar::STATUS_PAGAMENTO_BAIXADO);
                $boleto->persist();
            }
            self::updateBaixaReal($baixapaga,$nrmovbanc);


            // Se o tipo de baixa Contabilizar chama a Procedure de contabilização
            //(Rotina colocada no final da função pois o connection não da rollback na procedure de contabilização caso
            // ocorra algum erro durante a execução de outras rotinas).)
            if($baixapaga->getTpbaixapag()->getIdctblbxpag() == 'S'){

                self::contabilizaBaixaPaga($baixapaga->getCdempresa(),$cdfilial, $nrbaixapag );
                Registry::getEntityManager()->flush();
            }


        }catch(\Exception $e){
            throw $e;

        }

    }

    /**
     * @param Baixapaga $baixa
     * @param $nrmovbanc
     * @internal param $nrlancctbpag
     */
    private function updateBaixaReal(Baixapaga $baixa, $nrmovbanc){

        $baixa->setIdstatusbpag(Baixapaga::STATUS_EFETIVADA);
        $baixa->setNrmovbanc($nrmovbanc);
        $baixa->persist();
        Registry::getEntityManager()->flush();
    }

    public static function updateCheque($nrorg, $cdreducont, $nrcheque, $idstcheque, $dtemissaocheque, $valor = NULL, $dsnomcheq = NULL){
        /** @var Cheque $cheque */
        $cheque = Cheque::findOneByNrorgAndNrchequeAndCdreducont(
            $nrorg, $nrcheque, $cdreducont
        );
        $cheque->setIdstcheque($idstcheque);
        $dtemissaocheque = self::formataData($dtemissaocheque);
        $cheque->setDtemischeq($dtemissaocheque);
        $cheque->setDsnomcheq($dsnomcheq);
        $cheque->setDtmovbancheq($dtemissaocheque);
        $cheque->setVrcheque($valor);
        $cheque->persist();
        Registry::getEntityManager()->flush();
    }

    /**
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $dtfluxo
     * @param $idBaixa
     * @param $idModo
     * @throws \Exception
     * @internal param $vrbaixa
     */
    private function atualizaContas($cdfilial, $nrlanbolpag, $dtfluxo, $idBaixa, $idModo){
        try{

            $contas = ContaPagar::findByNrlanbolpagAndCdfilibpagAndNrorg($nrlanbolpag,
                                                                         $cdfilial, \Ambiente::getNrorg());

            $idtipo = $idBaixa == 'P' ? Movfluxo::TIPO_MOVIMENTO_EMPENHO : Movfluxo::TIPO_MOVIMENTO_REAL;

            /** @var $conta ContaPagar  */
            foreach($contas as $conta){
                if($idModo == 'I'){
                    $tipoBaixa = $idBaixa == 'P' ? 'P' :  'B';
                    $dtbaixa = $dtfluxo;
                }
                else{
                    $tipoBaixa = 'A';
                    $dtbaixa = $conta->getDtatuavenpag();
                    $idtipo = Movfluxo::TIPO_MOVIMENTO_EMPENHO;
                }

                $idfluxo = $conta->getIdmovfluxo() ? $conta->getIdmovfluxo() :
                                                     Movfluxo::TIPO_MOVIMENTO_EMPENHO;

                // Desfaz o movimento de fluxo estimado
                FluxodeCaixa::atualizaFluxo($conta->getCdempresa(),
                                            $conta->getCdfimovflpag(),
                                            $conta->getCdclassfina(),
                                            $conta->getDtmovfluxo(),
                                            $conta->getVrorigpag()*-1,
                                            $idfluxo,
                                            'SAIDA');

                // Inclui o movimento de fluxo real
                FluxodeCaixa::atualizaFluxo($conta->getCdempresa(),
                                            $conta->getCdfimovflpag(),
                                            $conta->getCdclassfina(),
                                            $dtbaixa,
                                            $conta->getVrorigpag(),
                                            $idtipo,
                                            'SAIDA');

                // Atualiza os campos referentes ao pagamento
                self::atualizaConta($conta,
                                    $idtipo,
                                    $dtbaixa,
                                    $tipoBaixa);

                $ajustes = Ajctpagar::findByCdfilialAndNrlanctonfAndNrseqvenpagAndNrparcelapagAndNrorg(
                                       $conta->getCdfilial(),$conta->getNrlanctonf(),$conta->getNrseqvenpag(),
                                       $conta->getNrparcelapag(), $conta->getNrorg());

                /** @var $ajuste Ajctpagar */
                if($ajustes){
                    foreach($ajustes as $ajuste){

                        if($idModo == 'E')
                            $dtbaixa = $ajuste->getDtajustepag();

                        $ajfluxo = $ajuste->getIdmovfluxo() ? $ajuste->getIdmovfluxo() : 'E';

                        if($ajuste->getAjuctpag()->getIdajuctpag() == \Geral\Entities\Tipoajpagar::ID_AJUSTE_ACRESCIMO){


                            // Desfaz o movimento de fluxo estimado
                            FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntp(),
                                                        $ajuste->getCdfimfajcntp(),
                                                        $ajuste->getCdclassfina(),
                                                        $ajuste->getDtmovfluxo(),
                                                        $ajuste->getVrajustepag() * -1,
                                                        $ajfluxo,
                                                        'SAIDA'
                                                        );

                            // Inclui o movimento de fluxo real
                            FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntp(),
                                                        $ajuste->getCdfimfajcntp(),
                                                        $ajuste->getCdclassfina(),
                                                        $dtbaixa,
                                                        $ajuste->getVrajustepag(),
                                                        $idtipo,
                                                        'SAIDA'
                                                        );
                        }
                        else{

                            // Desfaz o movimento de fluxo estimado
                            FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntp(),
                                                        $ajuste->getCdfimfajcntp(),
                                                        $ajuste->getCdclassfina(),
                                                        $ajuste->getDtmovfluxo(),
                                                        $ajuste->getVrajustepag(),
                                                        $ajfluxo,
                                                        'SAIDA'
                                                        );

                            // Inclui o movimento de fluxo real
                            FluxodeCaixa::atualizaFluxo($ajuste->getCdemmfajcntp(),
                                                        $ajuste->getCdfimfajcntp(),
                                                        $ajuste->getCdclassfina(),
                                                        $dtbaixa,
                                                        $ajuste->getVrajustepag() * -1,
                                                        $idfluxo,
                                                        'SAIDA'
                                                       );
                        }
                        self::atualizaAjuste($ajuste,$idtipo, $dtbaixa);

                    }
                }
            }
        }catch(\Exception $e){
            throw $e;

        }
    }

    /**
     * @param Ajctpagar $ajuste
     * @param $idmovfluxo
     * @param $dtmovfluxo
     */
    private function atualizaAjuste(Ajctpagar $ajuste, $idmovfluxo, DateTime $dtmovfluxo){

        $ajuste->setIdmovfluxo($idmovfluxo);
        $ajuste->setDtmovfluxo($dtmovfluxo);
        $ajuste->persist();
        Registry::getEntityManager()->flush();

    }


    /**
     * @param \Fiscal\Entities\Contapagar $conta
     * @param $idmovfluxo
     * @param \Setforms2\Shared\ORM\DateTime $dtmovfluxo
     * @param $idstatpag
     */
    private function atualizaConta(Contapagar $conta, $idmovfluxo,
                                   DateTime $dtmovfluxo, $idstatpag){

        $conta->setDtmovfluxo($dtmovfluxo);
        $conta->setIdstatpag($idstatpag);
        $conta->setIdmovfluxo($idmovfluxo);
        $conta->persist();
        Registry::getEntityManager()->flush();

    }

    //Atualiza o valor da baixa após deletar um boleto
    public static function atualizaBaixaPag($cdfilibaipag, $nrbaixapag, $nrorg, $vrfinalbpag = null, $nrmovbanc = null) {
        $db = \Setforms2\Runtime\Connection::getInstance();
        $db->getConn()->beginTransaction();
        $connection = \Setforms2\Shared\Registry::getEntityManager();

        try {
            $baixapaga = Baixapaga::findOneByCdfilialAndNrbaixapagAndNrorg($cdfilibaipag, $nrbaixapag, $nrorg);

            $baixapaga->setNrmovbanc($nrmovbanc);

            if($baixapaga->getVrbaixapag()) {
                $valorBaixa = $baixapaga->getVrbaixapag() - $vrfinalbpag;
                $baixapaga->setVrbaixapag($valorBaixa);
            }

            $baixapaga->persist();

            $connection->flush();
            $db->commit();

        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    //Atualiza o boleto
    public static function atualizaBoletoPag($cdfilial, $nrlanbolpag, $nrorg) {
        $db = \Setforms2\Runtime\Connection::getInstance();
        $db->getConn()->beginTransaction();
        $connection = \Setforms2\Shared\Registry::getEntityManager();

        try {
            $boleto = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($cdfilial, $nrlanbolpag, $nrorg);


            $boleto->setIdstatbpag('A');
            $boleto->setNrbaixapag(null);
            $boleto->setCdfilibaipag(null);
            $boleto->setDtliqbolpag(null);
            if($boleto->getNrcnabpagrem() == '')
                $boleto->setIdsitucnabp('00');
            else
                $boleto->setIdsitucnabp('10');

            $boleto->persist();


            $connection->flush();
            $db->commit();

        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    //Atualiza o status da Conta Pagar
    public static function atualizaStatusContaPagar($cdfilial, $nrlanbolpag, $nrorg, $idbaixa) {
        $db = \Setforms2\Runtime\Connection::getInstance();
        $db->getConn()->beginTransaction();
        $connection = \Setforms2\Shared\Registry::getEntityManager();

        try {
            /** @var $conta Contapagar */
            $contaPagar = Contapagar::findByCdfilibpagAndNrlanbolpagAndNrorg($cdfilial, $nrlanbolpag, $nrorg);


            if($idbaixa == 'E') {
                /** @var $conta Contapagar */
                foreach ($contaPagar as $conta) {
                    self::atualizaFluxodeCaixa($conta);
                    $conta->setIdstatpag('A');
                    $conta->persist();
                }
            }

            else{
                /** @var $conta Contapagar */
                foreach ($contaPagar as $conta) {
                    $conta->setIdstatpag('A');
                    $conta->persist();
                }
            }

            $connection->flush();
            $db->commit();

        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    public static function atualizaFluxodeCaixa($conta) {
        /** @var $conta Contapagar */

        FluxodeCaixa::atualizaFluxo($conta->getCdempresa(),$conta->getCdfimovflpag() ,$conta->getCdclassfina(), $conta->getDtmovfluxo(), -($conta->getVrorigpag()), $conta->getIdmovfluxo(), 'SAIDA');

        FluxodeCaixa::atualizaFluxo($conta->getCdempresa(),$conta->getCdfimovflpag() ,$conta->getCdclassfina(), $conta->getDtmovfluxo(), $conta->getVrorigpag(), 'E', 'SAIDA');
    }

    public static function deletaMovimentacao($nrorg, $cdreducont, $nrmovbanc_antigo)
    {
        $db = \Setforms2\Runtime\Connection::getInstance();
        $db->getConn()->beginTransaction();
        $connection = \Setforms2\Shared\Registry::getEntityManager();

        try {

            $movbanc = Movbancario::findOneByNrorgAndCdreducontAndNrmovbanc($nrorg, $cdreducont, $nrmovbanc_antigo);
            if($movbanc)
                $movbanc->delete();

            $connection->flush();
            $db->commit();

        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }

    }

    public static function atualizaBoletosBaixaPlanejada($boletos, $cdfilial, $nrbaixapag, $tipobaixa){
        $cont = count($boletos);
        for ($i = 0; $i < $cont; $i++) {
            /**
             * Método
             * @var $object Boletpagar
             */
            $object = Boletpagar::getObjectByWhere('NRORG = ? AND CDFILIAL = ? AND NRLANBOLPAG = ?', array($boletos[$i]['NRORG'], $boletos[$i]['CDFILIAL'], $boletos[$i]['NRLANBOLPAG']));
            $object->setNrbaixapag($nrbaixapag);
            $object->setIdstatbpag('P');
            $object->setCdfilibaipag($cdfilial);
            $object->setCdtpbaixapag($tipobaixa);
            $object->persist();
        }
        Registry::getEntityManager()->flush();
    }

    public static function atualizaContaBaixaPlanejada($boletos){
        $cont = count($boletos);
        for ($i = 0; $i < $cont; $i++) {
            /**
             * Método
             * @var $object Boletpagar
             */
            $object = Boletpagar::getObjectByWhere('NRORG = ? AND CDFILIAL = ? AND NRLANBOLPAG = ?', array($boletos[$i]['NRORG'], $boletos[$i]['CDFILIAL'], $boletos[$i]['NRLANBOLPAG']));
            /** @var $conta Contapagar */
            $contas = Contapagar::findByNrorgAndCdfilibpagAndNrlanbolpag($object->getNrorg(), $object->getCdfilial(), $object->getNrlanbolpag());
            foreach($contas as $conta) {
                $conta->setIdstatpag('P');
                $conta->persist();
            }
        }
        Registry::getEntityManager()->flush();
    }
}